from django.template import RequestContext
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext as _

from meetings.models import TeamMeeting
from teams.models import Team

from forms import TeamMeetingForm
from django.db.models import Q

from common.utils import redirect, simple_iterator
from common import launchpad

import datetime

def meeting_list(request):
    """
    a list with all meetings (teammeetings)
    """
    team_meeting_list = TeamMeeting.objects.next_meetings()
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    context = {
        'team_meeting_list': team_meeting_list,
	'user_is_on_lc': is_on_lc,
    }
    return render_to_response('meetings/meeting_list.html', context,
                  RequestContext(request))

def meeting_list_ical(meetings, name):
    """Return any list meetings as an ical"""
    filename = "%s.ics" % name.replace(' ', '-').lower()
    response = HttpResponse(mimetype='text/calendar')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename.encode('ascii', 'replace')
    response.write('''BEGIN:VCALENDAR
PRODID:-//loco.ubuntu.com//EN
VERSION:2.0
CALSCALE:GREGORIAN
METHOD:PUBLISH
X-WR-TIMEZONE:UTC
''')
    response.write('X-WR-CALNAME:%s\n' % name)
    response.write('X-WR-CALDESC:%s\n' % name)
    for meeting in meetings:
        response.write(event.as_ical())
    response.write('''END:VCALENDAR''')
    return response

def meeting_ical(request, team_meeting_id):
    """
    Return a ical list with a single meetings in ical format.
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    return meeting_list_ical([team_meeting_object], team_meeting_object.name)

def team_meeting_list_ical(request, team_slug):
    """
    Return a ical list with a single team meetings in ical format.
    """
    team = get_object_or_404(Team, lp_name=team_slug)
    team_meeting = TeamMeeting.objects.filter(teams__lp_name=team_slug)
    return meeting_list_ical(team_meeting, team.name)

def teams_meeting_list_ical(request):
    """
    Return a ical list with team meetings in ical format.
    """
    return meeting_list_ical(TeamMeetings.objects.all(), 'All Ubuntu Team Meetings')

def meeting_history_list(request):
    """
    a list with all historical meetings (teammeetings)
    """
    team_meeting_list = TeamMeeting.objects.history_meetings()
    context = {
        'team_meeting_list': team_meeting_list,
    }
    return render_to_response('meetings/meeting_history_list.html', context,
                  RequestContext(request))


#################################################################
# Team Meetings
#################################################################

def team_meeting_detail(request, team_meeting_id):
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    is_member = False
    for team_object in team_meeting_object.teams.all():
        is_member = is_member or launchpad.is_team_member(request.user, team_object)

    context = {
        'team_meeting_object': team_meeting_object,
        'user_is_team_member': is_member,
    }
    return render_to_response('meetings/team_meeting_detail.html', 
                                         context, RequestContext(request))

@login_required
def team_meeting_delete(request, team_meeting_id):
    """
    delete a meeting
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    #check if user is admin or owner of a team
    is_contact = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_admin_or_owner(request.user.username, team):
            is_contact = True
            break

    if is_on_lc or is_contact:
        if request.method == 'POST':		
            team_meeting_object.delete()
            request.user.message_set.create(message=_('Team Meeting removed.'))
            return redirect( 'meeting-list' )
        else:
            context = {'team_meeting_object': team_meeting_object}
            return render_to_response('meetings/team_meeting_delete_confirm.html', context, RequestContext(request))

    else:
        request.user.message_set.create(message=_('You can not remove this team meeting. You are not admin/owner of the Launchpad team or on the LoCo Council.'))
        return redirect( team_meeting_object )

@login_required
def team_meeting_select(request):
    teams = Team.objects.filter(lp_name__in=[g.name for g in request.user.groups.all()])
    if len(teams) == 0:
        request.user.message_set.create(message=_('You can not add a new team meeting. You are not member of any of the LoCo Teams.'))
        return redirect( 'meeting-list' )
    elif len(teams) == 1:
        from django.core import urlresolvers
        url = urlresolvers.reverse('team-meeting-new', args=[teams[0].lp_name])
        return HttpResponseRedirect(url)
    else:
        return render_to_response('meetings/team_meeting_new_select.html', context, RequestContext(request))

def team_meetings_rss(request, team_slug):
    team_object = get_object_or_404(Team, lp_name=team_slug)
    meetings = TeamMeeting.objects.filter(teams__lp_name=team_slug, date_end__gt=datetime.datetime.now()).order_by('date_begin')[:15]
    
    if request.META.has_key('wsgi.url_scheme'):
        scheme = request.META.get('wsgi.url_scheme')
    else:
        scheme = 'http'
        
    if request.META.has_key('HTTP_HOST'):
        host = request.META.get('HTTP_HOST')
    else:
        host = request.META.get('SERVER_NAME')
        
    if request.META.get('SERVER_PORT', 80) == 80 or request.META.get('SERVER_PORT', 80) == 0:
        base = '%s://%s' % (scheme, host)
    else:
        base = '%s://%s:%s' % (scheme, host, request.META.get('SERVER_PORT', 80))
    
    context = {'team_object': team_object, 'meetings': meetings, 'base':base}
    return render_to_response('meetings/team_meetings_rss.xml', context, RequestContext(request), mimetype='application/xhtml+xml')

@login_required
def team_meeting_new(request, team_slug):
    """
    new team meeting
    """
    team_object = get_object_or_404(Team, lp_name=team_slug)
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_member = launchpad.is_team_member(request.user, team_object)
    
    if is_on_lc or is_member:
        if request.method == 'POST':
            form = TeamMeetingForm(data=request.POST)
            if form.is_valid():
                team_meeting = form.save()
                team_meeting.teams.add(team_object)
                request.user.message_set.create(message=_('New meeting created.'))
                return redirect( team_object )
        else:
            form = TeamMeetingForm()
                
        context = {
            'team_object': team_object,
            'form': form,
        }
        return render_to_response('meetings/team_meeting_new.html', 
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message=_('You can not add a new meeting for this team. You are not member of the team or on the LoCo Council.'))
        return redirect( team_object )

@login_required
def team_meeting_copy(request, team_meeting_id):
    """
    copy team meeting
    """
    
    team_meeting = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    team_object = team_meeting.teams.all()[0]
    team_meeting.id = team_meeting.pk = None

    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    is_member = launchpad.is_team_member(request.user, team_object)
    
    if is_on_lc or is_member:
        if request.method == 'POST':
            form = TeamMeetingForm(instance=team_meeting, data=request.POST)
            if form.is_valid():
                team_meeting = form.save()
                team_meeting.teams.add(team_object)
                request.user.message_set.create(message=_('New meeting created.'))
                return redirect( team_object )
        else:
            form = TeamMeetingForm(instance=team_meeting)
        
        context = {
            'team_object': team_object,
            'form': form,
        }
        return render_to_response('meetings/team_meeting_new.html', 
                                         context, RequestContext(request))
    else:
        # XXX: Once we move to a new ACL system, this needs fixing.
        request.user.message_set.create(message=_('You can not add a new meeting for this team. You are not member of the team or on the LoCo Council.'))
        return redirect( team_object )

@login_required
def team_meeting_update(request, team_meeting_id):
    """
    update team meeting
    """
    team_meeting_object = get_object_or_404(TeamMeeting, pk=team_meeting_id)
    #check if user is admin or owner of a team
    is_member = False
    for team in team_meeting_object.teams.all():
        if launchpad.is_team_member(request.user, team):
            is_member = True
            break
    
    is_on_lc = launchpad.is_user_on_loco_council(request.user)
    
    if is_on_lc or is_member:
        if request.method == 'POST':		
            form = TeamMeetingForm(data=request.POST, instance=team_meeting_object)
            if form.is_valid():
                form.save()
                request.user.message_set.create(message=_('Team meeting updated.'))
                return redirect( team_meeting_object )
        else:
            form = TeamMeetingForm(instance=team_meeting_object)
            
        context = {
            'form': form,
        }
        return render_to_response('meetings/team_meeting_update.html', 
                                         context, RequestContext(request))
    else:
        request.user.message_set.create(message=_('You can not update this team meeting. You are not member of the team or on the LoCo Council.'))
        return redirect( team_meeting_object )
