from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.db.models import permalink

from teams.models import Country
import pytz


class VenueManager(models.Manager):
    """
    a manager class for Venues
    """
    pass 

class Venue(models.Model):
    """
    a venue
    """
    name = models.CharField(help_text=_('Name of the Venue'), max_length=150)
    country = models.ForeignKey(Country, null=True)
    spr = models.CharField(_("State/Province/Region"), max_length=50, null=True, blank=True)
    city = models.CharField(help_text=_('City Name'), max_length=150, null=True, blank=True)
    address = models.CharField(help_text=_('Address with Street and Number'), max_length=150, null=True, blank=True)
    longitude = models.FloatField(help_text=_('Longitude in Degrees East'), null=True, blank=True)
    latitude = models.FloatField(help_text=_('Latitude in Degrees North'), null=True, blank=True)
    tz = models.CharField(max_length=32, verbose_name=_('Timezone'), default='UTC', choices=[(tz, tz) for tz in pytz.all_timezones], blank=False, null=False)
    venue_url = models.URLField(help_text=_('URL for the Venue Homepage'), verbose_name=_('URL of the Venue'), max_length=200, verify_exists=False, blank=True, null=True)
    comment = models.TextField(help_text=_('Comment about the Venue'), blank=True, null=True)
    
    class Meta:
        unique_together = ( ('name', 'country', 'city'), ('longitude', 'latitude'), )

    #add manager 
    objects = VenueManager()

    def __unicode__(self):
        if self.spr and self.city:
            return '%(name)s (%(city)s, %(spr)s)' %{'name':self.name, 'city':self.city, 'spr':self.spr}
        if self.city:
            return '%(name)s (%(city)s)' %{'name':self.name, 'city':self.city}
        else:
            return self.name
    
    @models.permalink
    def get_absolute_url(self):
        """ get the absolute url for the venue """
        return ('venue-detail', [self.id])

    def get_timezone(self):
        try:
            return pytz.timezone(self.tz)
        except:
            return pytz.utc
    timezone = property(get_timezone)
    
    def tolocaltime(self, dt):
        as_utc = pytz.utc.localize(dt)
        return as_utc.astimezone(self.timezone)
    
    def fromlocaltime(self, dt):
        local = self.timezone.localize(dt)
        return local.astimezone(pytz.utc)
            
    def save(self, *args, **kargs):
        if self.id:
            from events.models import TeamEvent
            old = Venue.objects.get(id=self.id)
            retval = super(Venue, self).save(*args, **kargs)
            if old.tz != self.tz:
                events = TeamEvent.objects.filter(venue=self)
                for e in events:
                    old_offset = old.tolocaltime(e.date_begin).utcoffset()
                    new_offset = self.tolocaltime(e.date_begin).utcoffset()
                    e.date_begin = e.date_begin + (old_offset - new_offset)
                    e.date_end = e.date_end + (old_offset - new_offset)
                    e.save()
        else:
            retval = super(Venue, self).save(*args, **kargs)
        return retval
                
def venues_without_country():
    return Venue.objects.filter(country__isnull=True)
